Imports System.IO
Imports System.Data.SqlClient

#Const USE_DATABASE_SNAPSHOT = True
'
' Archive the SOSOS database into a Microsoft Access database file.  This
' program is designed to run as a scheduled task and therefore has no user
' interface.  The list of tables to include is in the config file.
'
Module Main
    Const COMMAND_TIMEOUT As Integer = 240

    Sub Main()
        Dim con As New SqlConnection
        Dim cmd As New SqlCommand
        Dim da As New SqlDataAdapter
        Dim dt, dts(0) As DataTable
        Dim table, msg As String
        Dim mdb_path, mdb_prev As String
        Dim el As New EventLog
        Dim stamp As Date
        Dim er, i, max_rows, row_count As Integer
        Dim db As New dt2mdb

        el.Source = "BackupSOSOS"

        Try
            msg = ""
            Console.WriteLine("BackupSOSOS")
            Console.WriteLine("Started: " & Now)
            stamp = Now

            ' Tack today's date onto the end of the file name?
            If My.Settings.FormatNameWithDate Then
                mdb_path = My.Settings.MDB_Path & "\" & My.Settings.Database_Name & "_" & Now.ToString("yyyyMMdd") & ".mdb"
            Else
                mdb_path = My.Settings.MDB_Path & "\" & My.Settings.Database_Name & ".mdb"
            End If

            ' Open the database now so there's no surprises later
            Try
                con.ConnectionString = My.Settings.Connection_String
                con.Open()
            Catch ex As Exception
                el.WriteEntry("Can't open the database" & vbCr & ex.Message, EventLogEntryType.Error)
                Beep()
                Console.Write("Can't open the database" & vbCrLf & ex.Message)
                System.Threading.Thread.Sleep(5000)
                Environment.ExitCode = 4313 'ERROR_DATABASE_FAILURE
                Exit Sub
            End Try

#If USE_DATABASE_SNAPSHOT Then
            ' delete and recreate a snapshot
            Try
                ' delete any previous snapshot
                cmd.Connection = con
                da.SelectCommand = cmd
                cmd.CommandText = "If Exists (SELECT name FROM sys.databases WHERE name = '" & My.Settings.SnapShotName & "') DROP DATABASE " & My.Settings.SnapShotName
                cmd.ExecuteNonQuery()

                ' create a new snapshot
                cmd.CommandText = "CREATE DATABASE " & My.Settings.SnapShotName & " ON ( NAME = '" & My.Settings.DatabaseLogicalName & "', FILENAME = '" & My.Settings.SnapShotPath & "') AS SNAPSHOT OF " & My.Settings.Database_Name & ";"
                cmd.ExecuteNonQuery()
            Catch ex As Exception
                el.WriteEntry("Can't create database snapshot" & vbCr & ex.Message, EventLogEntryType.Error)
                Beep()
                Console.Write("Can't create database snapshot" & vbCrLf & ex.Message)
                System.Threading.Thread.Sleep(5000)
                Environment.ExitCode = 4313 'ERROR_DATABASE_FAILURE
                Exit Sub
            End Try

            ' Pull a set of disconnected tables from the snapshot and write 'em to an MDB file
            con.Close()
            con.ConnectionString = My.Settings.SnapShotConnect
            con.Open()
#End If

            da.MissingSchemaAction = MissingSchemaAction.AddWithKey
            max_rows = My.Settings.MaxRows
            cmd.CommandTimeout = COMMAND_TIMEOUT
            For Each table In My.Settings.Table_List
                row_count = GetRowCount(cmd, table)
                dt = New DataTable
                dt.TableName = table
                dts(0) = dt
                cmd.CommandText = "Select * from [" & table & "]"
                Try
                    ' Since some tables can be quiet large, we break them up
                    ' into smaller chunks.
                    For i = 0 To row_count Step max_rows
                        Console.Write(".")
                        dts(0).Clear()
                        da.Fill(i, max_rows, dts)
                        If i = 0 Then
                            db.CopyToMDB(dts(0), mdb_path, My.Settings.EmptyTableRowsFirst)
                        Else
                            ' we obviously don't want to delete existing rows
                            ' during a partial fill operation!
                            db.CopyToMDB(dts(0), mdb_path, False)
                        End If
                    Next
                Catch ex As Exception
                    el.WriteEntry("Can't copy table " & table & vbCr & ex.Message & vbCr & ex.StackTrace, EventLogEntryType.Error)
                    Console.Write("!")
                    msg &= "Can't copy table " & table & vbCrLf
                    er += 1
                    ' not fatal
                End Try
            Next
            con.Close()

            ' Make a copy of the MDB file?
            Console.WriteLine("")
            If My.Settings.Make_Previous Then
                mdb_prev = My.Settings.MDB_Path & "\" & My.Settings.Database_Name & "_Previous.mdb"
                Console.WriteLine("Making a backup copy...")
                File.Copy(mdb_path, mdb_prev, True)
            End If

            ' Show 'em what we just did
            If msg <> "" Then
                Console.WriteLine(msg)
            End If
            Console.WriteLine("Finished: " & Now)
            System.Threading.Thread.Sleep(1000)

            ' also write it to the event log
            msg = "Started: " & stamp & vbCrLf & msg & "Finished: " & Now
            If er = 0 Then
                el.WriteEntry("Success!" & vbCr & msg, EventLogEntryType.Information)
            Else
                el.WriteEntry("Errors (" & er & ")" & vbCr & msg, EventLogEntryType.Warning)
            End If

        Catch ex As Exception
            el.WriteEntry("Internal Error" & vbCr & ex.Message & vbCr & ex.StackTrace, EventLogEntryType.Error)
            Beep()
            Console.WriteLine("Internal Error" & vbCrLf & ex.Message)
            System.Threading.Thread.Sleep(5000)
            Environment.ExitCode = 1287 ' ERROR_UNIDENTIFIED_ERROR
        End Try
    End Sub

    Public Function GetRowCount(ByVal cmd As SqlCommand, ByVal TableName As String) As Integer
        Dim ans As Integer

        cmd.CommandText = "Select count(*) from [" & TableName & "]"
        ans = CInt(cmd.ExecuteScalar)

        Return ans
    End Function
End Module
